// C Source File
// Created 2/9/2006; 4:31:34 PM

// Delete or comment out the items you do not need.
#define COMMENT_STRING         "Sudoku!"
#define COMMENT_PROGRAM_NAME   "Brute Force Sudoku Solver"
#define COMMENT_VERSION_STRING "0.1"
#define COMMENT_VERSION_NUMBER 0,1,0,0 /* major, minor, revision, subrevision */
#define COMMENT_AUTHORS        "sunjayc_99"

#include <tigcclib.h>

void StepDownGrid(int grid[9][3][3], int tgrid[9][9]);	// Steps down a 3D matrix to a 2D matrix
void StepDown(int i, int j, int k, int* n, int* m);		// Converts coordinates from 3D to 2D
void StepUpGrid(int tgrid[9][9], int grid[9][3][3]);	// Reverse of StepUpGrid
void StepUp(int n, int m, int* i, int* j, int* k);		// Reverse of StepUp
int Check(int grid[9][3][3]);							// Checks if nothing's conflicting with Sudoku rules
void Copy(int grid[9][3][3], int grid2[9][3][3]);		// Copies one grid to another
int Solve(int grid[9][3][3]);							// Recursive soving algorithm that I (sunjayc_99) created

// Main Function
void _main(void)
{	
	int key = 0;
	int x, y, oldx, oldy;
	int tgrid[9][9];
	int flag = 0;
	int n, m;
	int grid[9][3][3];
	int esc = KEY_ESC;
	int up = KEY_UP;
	int down = KEY_DOWN;
	int left = KEY_LEFT;
	int right = KEY_RIGHT;
	int enter = KEY_ENTER;
	
	ClrScr();				// Clear the screen
	FontSetSys(F_6x8);		// Set the font size
	for(n = 0; n < 9; n++)		// These nested loops draw the grid onto the screen
	{
		for(m = 0; m < 9; m++)
		{
			tgrid[n][m] = 0;
			if(n == 0 && m == 0)
			{
				DrawChar(n*10, m*10, '0', A_XOR);
				FontSetSys(F_8x10);
			}
			else
			{
				DrawChar(n*10, m*10, '0', A_XOR);
			}
		}
	}
	x = 0;
	y = 0;
	oldx = 0;
	oldy = 0;
	while(key != esc)			// Main input loop
	{
		if(kbhit())
		{
			key = ngetchx();
			if(key == up)			// Move up
			{
				if(y > 0)
				{
					oldx = x;
					oldy = y;
					y -= 1;
					flag = 1;
				}
			}
			if(key == down)			// Move down
			{
				if(y < 8)
				{
					oldx = x;
					oldy = y;
					y += 1;
					flag = 1;
				}
			}
			if(key == left)			// Move left
			{
				if(x > 0)
				{
					oldx = x;
					oldy = y;
					x -= 1;
					flag = 1;
				}
				else if(x == 0 && y > 0)	// Move to the next row
				{
					oldx = x;
					oldy = y;
					x = 8;
					y -= 1;
					flag = 1;
				}
			}
			if(key == right)			// Move right
			{
				if(x < 8)
				{
					oldx = x;
					oldy = y;
					x += 1;
					flag = 1;
				}
				else if(x == 8 && y < 8)	// Move to the previous row
				{
					oldx = x;
					oldy = y;
					x = 0;
					y += 1;
					flag = 1;
				}
			}
			if(key >= '0' && key <= '9')	// Number input
			{
				FontSetSys(F_6x8);
				DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
				tgrid[x][y] = key - '0';
				DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
			}
			if(key == enter)		//Solve it!
			{
				FontSetSys(F_6x8);
				DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
				FontSetSys(F_8x10);
				DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
				StepUpGrid(tgrid, grid);
				FontSetSys(F_4x6);
				MoveTo(110, 55);
				printf("Working...");
				if(Solve(grid) == -1)
				{
					clrscr();
					printf("Not enough memory");
					ngetchx();
					return;
				}
				StepDownGrid(grid, tgrid);
				FontSetSys(F_8x10);
				ClrScr();
				for(n = 0; n < 9; n++)
				{
					for(m = 0; m < 9; m++)
					{
						DrawChar(n*10, m*10, tgrid[n][m] + '0', A_XOR);
					}
				}
				key = KEY_ESC;
				ngetchx();
			}
		}
		if(flag == 1)				// The flag is set when an arrow key is pressed so we
		{							// know when to set the previous number to regular size
			FontSetSys(F_6x8);		// and the current number to small size
			DrawChar(oldx*10, oldy*10, tgrid[oldx][oldy] + '0', A_XOR);
			FontSetSys(F_8x10);
			DrawChar(oldx*10, oldy*10, tgrid[oldx][oldy] + '0', A_XOR);
			DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
			FontSetSys(F_6x8);
			DrawChar(x*10, y*10, tgrid[x][y] + '0', A_XOR);
			flag = 0;
		}
	}
}

void StepDownGrid(int grid[9][3][3], int tgrid[9][9])
{
	int i, j, k, n = 0, m = 0;

	for(i = 0; i < 9; i++)
	{
		for(j = 0; j < 3; j++)
		{
			for(k = 0; k < 3; k++)
			{
				n = ((i / 3) * 3) + j;
				m = ((i % 3) * 3) + k;
				tgrid[n][m] = grid[i][j][k];
			}
		}
	}
}

void StepDown(int i, int j, int k, int* n, int* m)
{
	*n = (i / 3) * 3;
	*m = (i % 3) * 3;
	*n += j;
	*m += k;
}

void StepUpGrid(int tgrid[9][9], int grid[9][3][3])
{
	int i, j, k, n = 0, m = 0;

	for(i = 0; i < 9; i++)
	{
		for(j = 0; j < 3; j++)
		{
			for(k = 0; k < 3; k++)
			{
				StepDown(i, j, k, &n, &m);
				grid[i][j][k] = tgrid[n][m];
			}
		}
	}
}

void StepUp(int n, int m, int* i, int* j, int* k)
{
	*i = (((n / 3) * 3) + (m / 3));
	*j = (n % 3);
	*k = (m % 3);
}

int Check(int grid[9][3][3])
{
	int i, j, k, n, m, e;
	int num[] = {1, 2, 3, 4, 5, 6, 7, 8, 9};
	int tgrid[9][9];

	e = 0;
	StepDownGrid(grid, tgrid);
	for(n = 0; n < 9; n++)
	{
		for(m = 0; m < 9; m++)
		{
			if(tgrid[n][m] != 0)
			{
				num[tgrid[n][m] - 1] = 0;
				for(i = 0; i < 9; i++)
				{
					if(tgrid[i][m] != 0 && i != n)
					{
						if(num[tgrid[i][m] - 1] == 0)
						{
							e += 1;
						}
					}
					if(tgrid[n][i] != 0 && i != m)
					{
						if(num[tgrid[n][i] - 1] == 0)
						{
							e += 1;
						}
					}
				}
				num[tgrid[n][m] - 1] = tgrid[n][m];
			}
		}
	}
	for(i = 0; i < 9; i++)
	{
		for(j = 0; j < 3; j++)
		{
			for(k = 0; k < 3; k++)
			{
				if(grid[i][j][k] != 0)
				{
					num[grid[i][j][k] - 1] = 0;
					for(n = 0; n < 3; n++)
					{
						for(m = 0; m < 3; m++)
						{
							if(grid[i][n][m] != 0 && n != j && m != k)
							{
								if(num[grid[i][n][m] - 1] == 0)
								{
									e += 1;
								}
							}
						}
					}
					num[grid[i][j][k] - 1] = grid[i][j][k];
				}
			}
		}
	}
	if(e)
	{
		return 0;
	}
	return 1;
}

void Copy(int grid[9][3][3], int grid2[9][3][3])
{
	int i, j, k;

	for(i = 0; i < 9; i++)
	{
		for(j = 0; j < 3; j++)
		{
			for(k = 0; k < 3; k++)
			{
				grid2[i][j][k] = grid[i][j][k];
			}
		}
	}
}

int Solve(int grid[9][3][3])
{
	int tmpgrid[9][3][3];
	int tgrid[9][9];
	int i, n, m, t, key = 0;

	if(!Check(grid))
	{
		return 0;
	}
	StepDownGrid(grid, tgrid);
	for(n = 0; n < 9; n++)
	{
		for(m = 0; m < 9; m++)
		{
			if(tgrid[n][m] == 0)
			{
				for(i = 1; i <= 9; i++)
				{
					key = kbhit();
					if(key == KEY_ESC)
					{
						return -1;
					}
					tgrid[n][m] = i;
					StepUpGrid(tgrid, tmpgrid);
					TRY
					t = Solve(tmpgrid);				// Makes sure that there's enough memory
					ONERR
					{
						return -1;
					}
					ENDTRY
					if(t == 1)
					{
						Copy(tmpgrid, grid);
						return 1;
					}
					else if(t == -1)
					{
						return -1;
					}
				}
				return 0;
			}
		}
	}
	return 1;
}